// WARNING
// before printing verify that filament that will be use can
// withstand temperature form the hardware and will not melt

/* 
 * Enclosure for Olimex A20-OLinuXino-LIME2 
 * Copyright (c) 2019 Skoda
 * Author is not in any way associated with the Olimex Ltd
 *
 * This code is multi-licensed under GNU General Public License
 * version 3 of the License, or  (at your option) any later version
 * and under the Creative Commons Attribution-ShareAlike 4.0 
 * International.
 * You can choose to use any of these licenses as you see fit,
 * although keeping the multi-license is recommended.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Creative Commons Attribution-ShareAlike 4.0 
 * International license can be found at
 * https://creativecommons.org/licenses/by-sa/4.0/legalcode
 */





//Edit these parameters to customize the box

w = 2; //defualt 2
x = 86 + 2*w;  //defualt 86 +  2*w
y = 63 + 2*w; //default 63 +  2*w
z = 25  + 2*w; //default 25 +  2*w

///Parameters recommendations

//WALL THICKNESS
// more than 2 mm
//using a wall thickness that is a multiple of the nozzle size will result in a much better print. If 0.4mm nozzle is going to be used to print this part so a wall thickness that was 6 x the nozzle size was selected (0.4mm x 6 = 2.4mm).

//CLEARANCE
// add extra 0.5 mm to components size
// add 1 mm on each side of of port wall 
// more tips https://www.3dhubs.com/knowledge-base/enclosure-design-3d-printing-step-step-guide





//Ports positions
//P - port, U - upper box, L - lower box, numbering from right to left - rising with the y axis
//"space" says where the between ports is
//e.g. PL1upper_space_PL2 defines space between  ports PL1upper and PL2

space_PL1upper = 5.35 - 2;
PL1upper_z = 5 + 3.2;

space_PL1lower = 2.84 - 2;
PL1lower_z = 1 + 3.2;

PL1upper_length = 8.25;
PL1upper_height = 4.41;

PL1lower_length = 15.40;
PL1lower_height = 2.30;

PL1upper_space_PL2 = 5.80;

PL2_z = 7.1 + 1.4;
PL2_length = 15.50;
PL2_height = 6.91;

PL2_space_PL3 = 0.88;

PL3_z = 7 + 1.4;
PL3_length = 24.25;
PL3_height = 4.50;

PL3left_length = 5.20;
PL3left_height = 6.70;

//upper

// "+ w" as upper part is "+w" higher (otherwise it wouldn't snap together with lower part)
PU_z = 6.69 - 2.5 + w;


space_PU1 = 5;

PU1_length = 16.77;
PU1_height = 14.22 + 0.5;

PU1_space_PU2 = 1.80;

PU2_length = 7.63;
PU2_height = 15.04 + 0.5;

PU2_space_PU2 = 1.80;

PU2_space_PU3 = 0.60;

PU3_length = 16.50;
PU3_height = 5.52  + 0.5;

PU3left_length = 9.55;
PU3left_height = 11.48  + 0.5;

//set to achieve oval shape of the all holes
$fa=0.5; // default minimum facet angle is now 0.5
$fs=0.5; // default minimum facet size is now 0.5 mm

////embedded nut  https://www.3dhubs.com/knowledge-base/how-assemble-3d-printed-parts-threaded-fasteners#embedding-a-nut
Fastner_HoleDepth = 2.34; //plastic 2.34; metal: 2.77
Fastner_CubeSize = 9; //size + 1 in overlaies with the box, both holes are moved +1 to put them further form the box
FastnerCube_x = 9;
FastnerCube_y = 9;
FastnerCube_z = 5;
Fastner_LowerHoleRadius = 5.46/2; //plastic 5.86; metal: 5.46 
Fastner_UpperHoleRadius = 2.96/2; //plastic 2.80; metal: 2.84

//Holes for fixing board's position
//defines holes for the screws
 
//higher - goes up on the x axis
//right - goes to zero on the y axis
//distance from nearest edge of the wall, without the wall thickness

RightHoles_y = 5.19;
LeftHoles_y = 5.19;
LowerHoles_x = 25.5;
UpperHoles_x = 17.55;
Holes_r = 2.96/2;





//Defining modules of shapes that are reused later

//Creates the box - cuboid with hollow insides
module box(x_BoxExterior, y_BoxExterior, z_BoxExterior, WallWidht, x_NoWall = 0, x_NoStartWall = 0, y_NoWall = 0, y_NoStartWall = 0, z_NoWall = 0, z_NoStartWall = 0,)
{
    //this craves the inside of the box
    difference()
    {

        //exterior dimension of the box
        cube([x_BoxExterior,y_BoxExterior,z_BoxExterior], center = false ); 

        //this defines the space that will be subtracted, creates the "subtracting  cube"
        translate ([
            (1 - x_NoStartWall)*WallWidht - x_NoStartWall,
            (1 - y_NoStartWall)*WallWidht - y_NoStartWall,
            (1 - z_NoStartWall)*WallWidht - z_NoStartWall
        ])
        //if {axis}_NoStartWall = 0 the substracting cube won't start at zero, but in WallWidth position further, creating so wall of WallWidth length, {axis}_NoStartWall is substracted that is substraction is neutralized as {axis}_NoStartWall = 0 
        //if {axis}_NoStartWall = 0 the subtracting  cube won't start at zero, creating so no wall, subtracting minus one through "- {axis}_NoStartWall = 1" cut through the cube and does not leave surface of zero width
        {
            cube([
            x_BoxExterior + x_NoWall - (2 -  x_NoWall)*WallWidht,
            y_BoxExterior + y_NoWall - (2 -  y_NoWall)*WallWidht,
            z_BoxExterior + z_NoWall - (2 -  z_NoWall)*WallWidht
            ], center = false);
            //these lines set properties of the subtracting box
            //if "{axis}_NoWall = 0" subtracting box "- 2*WallWidht" shorter than original box, creating walls
            // /if "{axis}_NoWall = 1" subtracting box "- 1*WallWidht" shorter leaving space for one walls
            //if "{axis}_NoWall = 2"  subtracting is as long as the original box on this {axis}, crating no walls
          
        }
    }
  }
 //zero walls are deleted {axis}_NoWall = 0, {axis}_NoStartWall = 0
 //one wall startig at [0,0,0] parallel with the {axis} is deleted {axis}_NoWall = 1, {axis}_NoStartWall = 1,
 //one wall ending at [x_BoxExterior, y_BoxExterior, z_BoxExterior] parallel with the {axis} is deleted {axis}_NoWall = 1, {axis}_NoStartWall = 1
 //two walls parallel with the {axis} are deleted {axis}_NoWall = 2, {axis}_NoStartWall = 1
  
  //use to "drill holes" used for ports 
  module hole(length_Hole, height_Hole, y_Hole, z_Hole, x_BoxExterior, WallWidht, EndWall = 0, )
{  
    translate([EndWall*(x_BoxExterior - WallWidht) - 1 , WallWidht + y_Hole, WallWidht + z_Hole])
  // "WallWidht +" ensures that hole is not cut in the neigbouring walls
  {cube([WallWidht + 2, length_Hole, height_Hole], center = false);}
  // "WallWidht + 2" ensures that it cuts through the wall
 // it is "+2" and not "+1" while translate it moves -1 
// module hole, cut hole 1 for back, 0 for front
}
//EndWall if 0, hole module cuts a hole in the starting [0,0] wall, if 1 it cuts hole int the [x_BoxExterior, 0] wall




//Modules and parameters are used here to create the final product

// lower part of the box
union()
{
difference()
{
    box(x_BoxExterior = x, y_BoxExterior = y, z_BoxExterior = z, WallWidht = w,  x_NoWall = 1, y_NoWall =0,  z_NoWall = 1);
    {     
        // defines the holes for the ports
        hole(length_Hole = PL1upper_length, height_Hole = PL1upper_height, y_Hole = space_PL1upper, z_Hole = PL1upper_z, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
          hole(length_Hole = PL1lower_length, height_Hole = PL1lower_height, y_Hole = space_PL1lower, z_Hole = PL1lower_z, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
         hole(length_Hole = PL2_length, height_Hole = PL2_height, y_Hole =  space_PL1upper + PL1upper_length + PL1upper_space_PL2 , z_Hole = PL2_z, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
        hole(length_Hole = PL3_length, height_Hole = PL3_height, y_Hole =  space_PL1upper + PL1upper_length + PL1upper_space_PL2 + PL2_length + PL2_space_PL3, z_Hole = PL3_z, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
          hole(length_Hole = PL3left_length, height_Hole = PL3left_height, y_Hole = space_PL1upper + PL1upper_length + PL1upper_space_PL2 + PL2_length + PL2_space_PL3 + (PL3_length - PL3left_length) , z_Hole = PL3_z, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
          hole(length_Hole = 2.85, height_Hole = 1.6, y_Hole = space_PL1upper + PL1upper_length + PL1upper_space_PL2 + PL2_length + PL2_space_PL3 + 2.6, z_Hole = PL3_z - 1.3 - 1.6, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
         hole(length_Hole = 2.85, height_Hole = 1.6, y_Hole = space_PL1upper + PL1upper_length + PL1upper_space_PL2 + PL2_length + PL2_space_PL3 + 2.6 + 2.85 + 5.6, z_Hole = PL3_z - 1.3 - 1.6, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
         hole(length_Hole = 2.85, height_Hole = 1.6, y_Hole = space_PL1upper + PL1upper_length + PL1upper_space_PL2 + PL2_length + PL2_space_PL3 + 2.6 + 2.85 + 5.6 + 2.85 + 5.6, z_Hole = PL3_z - 1.3 - 1.6, x_BoxExterior = x, WallWidht = w, EndWall = 0);
        
         //ScrewHoles_x = 22.12 + w ; x - 21
        //RightHoles_y = 4.19 + w; 

        //defines holes for the screws
        
        //higher - goes up on the x axis
        //right - goes to zero on the y axis             
        
        //lower-right corner 
        translate([LowerHoles_x  + w, RightHoles_y + w,0]) {cylinder(h=2*w + 2, r=2.96/2, center=true);} 
        
        //upper-right corner
       translate([x - (UpperHoles_x + w), RightHoles_y + w,0]) {cylinder(h=2*w + 2, r=2.96/2, center=true);}
        
       //lower-left corner
       translate([LowerHoles_x + w,y - (LeftHoles_y + w),0]) {cylinder(h=2*w + 2, r=2.96/2, center=true);}
       
       //upper-left corner 
       translate([x - (UpperHoles_x + w),y - (LeftHoles_y + w),0]) {cylinder(h=2*w + 2, r=2.96/2, center=true);}
    }
}

translate([w - 1, y/2 - FastnerCube_y/2, z - FastnerCube_z])
{
    difference()
    {
        cube(size = [FastnerCube_x + 1, FastnerCube_y, FastnerCube_z],   center = false);
        
        
    
        {
             translate([(FastnerCube_x  + 1) /2,FastnerCube_y/2, -1])
            {
            cylinder(h=FastnerCube_z  + 2, r=Fastner_UpperHoleRadius, center=false);
            }
            
            translate([(FastnerCube_x + 1)/2,FastnerCube_y/2, -1])
            {    
            cylinder(h=Fastner_HoleDepth  + 1, r=Fastner_LowerHoleRadius, center=false, $fn = 6);
            }
        }
    }
}

}


// upper part of the box
translate([x + 20, 0, 0])
{
    difference()
    {    
        // adding width w to length x and height z so it will snap with the lower part of the box
        //"x + w" raises up the point where the height ends, so we added "+ w" to z_Hole in hole() function
        //"z + w" takes the end of the length +w further, so we added + w to x_BoxExterior in hole() function
        box(x_BoxExterior = x + w, y_BoxExterior = y, z_BoxExterior = z + w, WallWidht = w, x_NoWall = 1, x_NoStartWall = 1, y_NoWall = 2, y_NoStartWall = 1, z_NoWall = 1, z_NoStartWall = 1);
        
        {
                    // this defines the holes for the ports
            
            hole(length_Hole = PU1_length, height_Hole = PU1_height, y_Hole = space_PU1, z_Hole = PU_z, x_BoxExterior = x + w, WallWidht = w, EndWall = 1);
            
            hole(length_Hole = PU2_length , height_Hole = PU2_height, y_Hole = space_PU1 + PU1_length  + PU2_space_PU2, z_Hole = PU_z, x_BoxExterior = x  + w, WallWidht = w, EndWall = 1);
            
             hole(length_Hole = PU2_length , height_Hole = PU2_height, y_Hole = space_PU1 + PU1_length  + PU1_space_PU2 + PU2_length  + PU2_space_PU2 , z_Hole = PU_z, x_BoxExterior = x  + w, WallWidht = w, EndWall = 1);
        
        hole(length_Hole = PU3_length, height_Hole = PU3_height, y_Hole = space_PU1 + PU1_length  + PU1_space_PU2 + PU2_length + PU2_space_PU2 + PU2_length + PU2_space_PU3 , z_Hole = PU_z, x_BoxExterior = x  + w, WallWidht = w, EndWall = 1);
            
           hole(length_Hole = PU3left_length, height_Hole = PU3left_height, y_Hole = space_PU1 + PU1_length  + PU1_space_PU2 + PU2_length + PU2_space_PU2  + PU2_length + PU2_space_PU3 + (PU3_length  - PU3left_length), z_Hole = PU_z, x_BoxExterior = x + w, WallWidht = w, EndWall = 1);
            
            // defines hole for Threaded Fasteners
            
            translate([(FastnerCube_x/2) + w - 1,y/2, z - 1])
            {
          cylinder(h=w  + 2, r=Fastner_UpperHoleRadius, center=false);
            }
            
        }
    }
 }
 
 
 // Ideas for improvements and alternative solutions
 
 //Alternative Enclosure mechanism
//Lock block
    //CC-BY OpenScad https://www.thingiverse.com/thing:27695
// Interlocking joints
    //(locking button) http://www.mkrclub.com/2015/02/experimenting-with-plastic-joints.html
//Enclosure design tips https://www.3dhubs.com/knowledge-base/enclosure-design-3d-printing-step-step-guide


//Export EeagleCAD/KiCAD Lime2 3D model to OpenSCAD fro more accurate measuremetns
//Print form EeagleCAD https://rayshobby.net/eaglecad-script-for-laser-cut-enclosuresrs